# -*- coding: utf-8 -*-
#
# gPodder - A media aggregator and podcast client
# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
#
# gPodder is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# gPodder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pango
import threading
import hildon
import urllib
import gobject

import gpodder

_ = gpodder.gettext
N_ = gpodder.ngettext

from gpodder import util
from gpodder import opml
from gpodder import youtube

from gpodder.gtkui.frmntl.opml import OpmlListModel

from gpodder.gtkui.interface.common import BuilderWidget

from gpodder.gtkui.frmntl.widgets import EditToolbarDeluxe

from gpodder.gtkui.draw import draw_text_box_centered

class gPodderPodcastDirectory(BuilderWidget):
    def new(self):
        self._is_updating = False

        if hasattr(self, 'custom_title'):
            self.main_window.set_title(self.custom_title)

        if not hasattr(self, 'add_urls_callback'):
            self.add_urls_callback = None

        titlecell = gtk.CellRendererText()
        titlecell.set_property('ellipsize', pango.ELLIPSIZE_END)
        titlecolumn = gtk.TreeViewColumn('', titlecell, markup=OpmlListModel.C_DESCRIPTION_MARKUP)
        self.treeview.append_column(titlecolumn)

        selection = self.treeview.get_selection()
        selection.connect('changed', self.on_selection_changed)
        selection.set_mode(gtk.SELECTION_MULTIPLE)
        selection.unselect_all()
        self.app_menu = hildon.AppMenu()
        for action in (self.action_select_all, \
                       self.action_select_none):
            button = gtk.Button()
            action.connect_proxy(button)
            self.app_menu.append(button)
        self.main_window.set_app_menu(self.app_menu)

        self.edit_toolbar = EditToolbarDeluxe(self.main_window.get_title(), \
                _('Subscribe'))
        self.edit_toolbar.connect('arrow-clicked', \
                self.on_close_button_clicked)
        self.edit_toolbar.connect('button-clicked', \
                self.on_subscribe_button_clicked)
        self.edit_toolbar.show_all()

        # This method needs a EditToolbarDeluxe to work
        self.edit_toolbar.set_button_sensitive(False)

        self.main_window.set_edit_toolbar(self.edit_toolbar)
        self.main_window.fullscreen()
        self.main_window.show()

    @classmethod
    def show_add_podcast_picker(cls, parent, toplist_url, opml_url, \
            add_urls_callback, subscribe_to_url_callback, \
            my_gpodder_callback, show_text_edit_dialog):
        dialog = gtk.Dialog(_('Select a source'), parent)
        pannable_area = hildon.PannableArea()
        pannable_area.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)
        dialog.vbox.pack_start(pannable_area, expand=True)
        vbox = gtk.VBox(spacing=1)
        pannable_area.add_with_viewport(vbox)

        def load_opml_from_url(url):
            if url is not None:
                o = cls(parent, add_urls_callback=add_urls_callback)
                o.download_opml_file(url)

        def choice_enter_feed_url(widget):
            dialog.destroy()
            subscribe_to_url_callback()

        def choice_load_opml_from_url(widget):
            dialog.destroy()
            url = show_text_edit_dialog(_('Load OPML file from the web'), \
                    _('URL:'), is_url=True, affirmative_text=_('Load'))
            load_opml_from_url(url)

        def choice_search_mygpo(widget):
            dialog.destroy()
            search_term = show_text_edit_dialog(\
                    _('Search on gpodder.net'), \
                    _('Search for:'), affirmative_text=_('Search'))
            if search_term is not None:
                url = 'http://gpodder.net/search.opml?q=%s' % (urllib.quote(search_term),)
                load_opml_from_url(url)

        def choice_load_opml_from_file(widget):
            dialog.destroy()
            dlg = gobject.new(hildon.FileChooserDialog, \
                    action=gtk.FILE_CHOOSER_ACTION_OPEN)
            dlg.set_title(_('Open OPML file'))
            dlg.show_all()
            dlg.run()
            filename = dlg.get_filename()
            dlg.hide()
            if filename is not None:
                load_opml_from_url(filename)

        def choice_load_examples(widget):
            dialog.destroy()
            load_opml_from_url(opml_url)

        def choice_load_toplist(widget):
            dialog.destroy()
            load_opml_from_url(toplist_url)

        def choice_search_youtube(widget):
            dialog.destroy()
            search_term = show_text_edit_dialog(\
                    _('Search YouTube user channels'), \
                    _('Search for:'), affirmative_text=_('Search'))
            if search_term is not None:
                url = 'youtube://%s' % (search_term,)
                load_opml_from_url(url)

        def choice_mygpodder(widget):
            dialog.destroy()
            my_gpodder_callback()

        choices = (
                (_('Podcast feed/website URL'), choice_enter_feed_url),
                (_('OPML file from the web'), choice_load_opml_from_url),
                (_('Search on gpodder.net'), choice_search_mygpo),
                (_('Open OPML file'), choice_load_opml_from_file),
                (_('Example podcasts'), choice_load_examples),
                (_('Podcast Top 50'), choice_load_toplist),
                (_('Search YouTube users'), choice_search_youtube),
                (_('Download from gpodder.net'), choice_mygpodder),
        )

        for caption, handler in choices:
            button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | \
                    gtk.HILDON_SIZE_FINGER_HEIGHT, \
                    hildon.BUTTON_ARRANGEMENT_VERTICAL)
            button.set_text(caption, '')
            button.connect('clicked', handler)
            vbox.pack_start(button)

        dialog.show_all()

    def on_treeview_expose_event(self, treeview, event):
        if event.window == treeview.get_bin_window():
            model = treeview.get_model()
            if (model is not None and model.get_iter_first() is not None):
                return False

            ctx = event.window.cairo_create()
            ctx.rectangle(event.area.x, event.area.y,
                    event.area.width, event.area.height)
            ctx.clip()
            x, y, width, height, depth = event.window.get_geometry()

            if self._is_updating:
                text = _('Loading podcast list')
            else:
                text = _('No podcasts')

            from gpodder.gtkui.frmntl import style
            font_desc = style.get_font_desc('LargeSystemFont')
            draw_text_box_centered(ctx, treeview, width, height, text, font_desc)

        return False


    def on_selection_changed(self, selection):
        self.set_subscribe_button_sensitive()

    def get_selected_channels(self):
        selection = self.treeview.get_selection()
        model, paths = selection.get_selected_rows()
        return [model.get_value(model.get_iter(path), \
                OpmlListModel.C_URL) for path in paths]

    def download_opml_file(self, url):
        selection = self.treeview.get_selection()
        selection.unselect_all()
        self.treeview.set_model(None)
        self._is_updating = True
        self.treeview.queue_draw()
        hildon.hildon_gtk_window_set_progress_indicator(self.main_window, True)

        def download_thread_func():
            if url.startswith('youtube://'):
                importer = youtube.find_youtube_channels(\
                        url[len('youtube://'):])
            else:
                importer = opml.Importer(url)

            if importer.items:
                model = OpmlListModel(importer)
            else:
                model = None
            def download_thread_finished():
                self._is_updating = False
                self.treeview.queue_draw()
                hildon.hildon_gtk_window_set_progress_indicator(\
                        self.main_window, False)
                self.action_select_all.set_property('visible', \
                        model is not None)
                self.action_select_none.set_property('visible', \
                        model is not None)
                self.treeview.set_model(model)
                self.set_subscribe_button_sensitive()

                if model is None:
                    self.show_message(_('No podcasts found. Try another source.'), \
                            important=True)
                    self.main_window.destroy()

            util.idle_add(download_thread_finished)

        threading.Thread(target=download_thread_func).start()

    def on_select_all_button_clicked(self, widget):
        selection = self.treeview.get_selection()
        selection.select_all()

    def on_select_none_button_clicked(self, widget):
        selection = self.treeview.get_selection()
        selection.unselect_all()

    def set_subscribe_button_sensitive(self):
        selection = self.treeview.get_selection()
        title = [self.main_window.get_title()]
        if selection:
            count = selection.count_selected_rows()
            text = N_('%(count)d podcast selected', '%(count)d podcasts selected', count)
            title.append(text % {'count': count})
        else:
            count = 0
        self.edit_toolbar.set_label(' - '.join(title))
        self.edit_toolbar.set_button_sensitive(count > 0)

    def on_subscribe_button_clicked(self, widget, *args):
        channel_urls = self.get_selected_channels()
        self.main_window.destroy()

        # add channels that have been selected
        if self.add_urls_callback is not None:
            self.add_urls_callback(channel_urls)

    def on_close_button_clicked(self, widget):
        self.main_window.destroy()




# -*- coding: utf-8 -*-
#
# gPodder - A media aggregator and podcast client
# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
#
# gPodder is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# gPodder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

try:
    import gobject
    import pygst
    pygst.require('0.10')
    import gst
    from gst.extend.discoverer import Discoverer
    have_gst = True
except:
    have_gst = False

class GstFile:
    def __init__(self):
        self.mainloop = gobject.MainLoop()
        self.result = None

    def run(self, filename):
        gobject.idle_add(self.on_idle, filename)
        self.mainloop.run()
        if self.result is not None:
            return self.result / gst.MSECOND
        else:
            return None

    def on_idle(self, filename):
        d = Discoverer(filename)
        d.connect('discovered', self.on_data)
        d.discover()
        return False

    def on_data(self, discoverer, ismedia):
        if discoverer.is_video:
            self.result = discoverer.videolength
        elif discoverer.is_audio:
            self.result = discoverer.audiolength
        gobject.idle_add(self.mainloop.quit)

def get_track_length(filename):
    """
    Returns track length in microseconds.

    Prefers video streams to audio streams. If no supported streams were found,
    returns None.
    """
    if not have_gst:
        return None
    return GstFile().run(filename)

__all__ = ['get_track_length']

# vim: set ts=4 sts=4 sw=4 et:



# -*- coding: utf-8 -*-
#
# gPodder - A media aggregator and podcast client
# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
#
# gPodder is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# gPodder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


# Run Doctests and Unittests for gPodder modules
# 2009-02-25 Thomas Perl <thp@gpodder.org>


import doctest
import unittest
import sys

try:
    # Unused here locally, but we import it to be able to give an early
    # warning about this missing dependency in order to avoid bogus errors.
    import minimock
except ImportError, e:
    print >>sys.stderr, """
    Error: Unit tests require the "minimock" module (python-minimock).
    Please install it before running the unit tests.
    """
    sys.exit(2)

# Which package and which modules in the package should be tested?
package = 'gpodder'
modules = ['util']
coverage_modules = []

suite = unittest.TestSuite()

for module in modules:
    m = __import__('.'.join((package, module)), fromlist=[module])
    coverage_modules.append(m)
    suite.addTest(doctest.DocTestSuite(m))

runner = unittest.TextTestRunner(verbosity=2)

try:
    import coverage
except ImportError:
    coverage = None

if coverage is not None:
    coverage.erase()
    coverage.start()

result = runner.run(suite)

if not result.wasSuccessful():
    sys.exit(1)

if coverage is not None:
    coverage.stop()
    coverage.report(coverage_modules)
    coverage.erase()

if coverage is None:
    print >>sys.stderr, """
    No coverage reporting done (Python module "coverage" is missing)
    Please install the python-coverage package to get coverage reporting.
    """




# -*- coding: utf-8 -*-
#
# gPodder - A media aggregator and podcast client
# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
#
# gPodder is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# gPodder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pango
import threading
import urllib
import hildon

import gpodder

_ = gpodder.gettext

from gpodder import util
from gpodder import opml
from gpodder import youtube

from gpodder.gtkui.opml import OpmlListModel

from gpodder.gtkui.interface.common import BuilderWidget


class gPodderPodcastDirectory(BuilderWidget):
    finger_friendly_widgets = ('button_cancel', 'button_subscribe')
    
    def new(self):
        if hasattr(self, 'custom_title'):
            self.main_window.set_title(self.custom_title)

        if not hasattr(self, 'add_urls_callback'):
            self.add_urls_callback = None

        togglecell = gtk.CellRendererToggle()
        togglecolumn = gtk.TreeViewColumn('', togglecell, active=OpmlListModel.C_SELECTED)
        self.treeview.append_column(togglecolumn)

        titlecell = gtk.CellRendererText()
        titlecell.set_property('ellipsize', pango.ELLIPSIZE_END)
        titlecolumn = gtk.TreeViewColumn('', titlecell, markup=OpmlListModel.C_DESCRIPTION_MARKUP)
        self.treeview.append_column(titlecolumn)

        self.treeview.connect('button-release-event', \
                self.on_treeview_button_release)

        menu = gtk.Menu()
        item = gtk.MenuItem(_('Load podcast list'))
        submenu = gtk.Menu()
        submenu.append(self.action_load_opml.create_menu_item())
        submenu.append(self.action_load_toplist.create_menu_item())
        submenu.append(self.action_search_mygpo.create_menu_item())
        submenu.append(self.action_load_youtube.create_menu_item())
        item.set_submenu(submenu)
        menu.append(item)
        menu.append(gtk.SeparatorMenuItem())
        menu.append(self.action_select_all.create_menu_item())
        menu.append(self.action_select_none.create_menu_item())
        menu.append(gtk.SeparatorMenuItem())
        menu.append(self.action_invert_selection.create_menu_item())
        menu.append(gtk.SeparatorMenuItem())
        menu.append(self.action_close.create_menu_item())
        self.main_window.set_menu(self.set_finger_friendly(menu))
        self.main_window.connect('key-press-event', self._on_key_press_event)

    def _on_key_press_event(self, widget, event):
        if event.keyval == gtk.keysyms.Escape:
            self.on_close_button_clicked(widget)
            return True
        else:
            return False

    def on_selection_changed(self, selection):
        self.set_subscribe_button_sensitive()

    def on_treeview_button_release(self, widget, event):
        selection = widget.get_selection()
        model, iter = selection.get_selected()
        if iter is not None:
            model.set_value(iter, OpmlListModel.C_SELECTED, \
                    not model.get_value(iter, OpmlListModel.C_SELECTED))
        self.set_subscribe_button_sensitive()

    def get_selected_channels(self):
        model = self.treeview.get_model()
        if model is not None:
            return [row[OpmlListModel.C_URL] for row in model \
                    if row[OpmlListModel.C_SELECTED]]

        return []

    def on_load_opml_button_clicked(self, widget):
        url = self.show_text_edit_dialog(_('Load OPML file from the web'), _('URL:'))
        if url is not None:
            self.download_opml_file(url)
    
    def on_load_toplist_button_clicked(self, widget):
        self.download_opml_file(self._config.toplist_url)

    def on_search_mygpo_button_clicked(self, widget):
        search_term = self.show_text_edit_dialog(\
                _('Search on gpodder.net'), \
                _('Search for:'))
        if search_term is not None:
            self.download_opml_file('http://gpodder.net/search.opml?q=%s' % ( \
                    urllib.quote(search_term),))

    def on_load_youtube_button_clicked(self, widget):
        search_term = self.show_text_edit_dialog(\
                _('Search YouTube user channels'), \
                _('Search for:'))
        if search_term is not None:
            self.download_opml_file(search_term, use_youtube=True)
    
    def download_opml_file(self, url, use_youtube=False):
        self.treeview.get_selection().unselect_all()
        self.treeview.set_sensitive(False)
        self.button_subscribe.set_sensitive(False)

        self.button_cancel.set_sensitive(False)
        banner = hildon.hildon_banner_show_animation(self.main_window, \
                '', _('Loading podcast list, please wait'))

        def download_thread_func():
            if use_youtube:
                importer = youtube.find_youtube_channels(url)
            else:
                importer = opml.Importer(url)

            if importer.items:
                model = OpmlListModel(importer)
            else:
                model = None
                self.show_message(_('Please pick another source.'), _('No podcasts found'))

            def download_thread_finished():
                if banner is not None:
                    banner.destroy()
                self.treeview.set_model(model)
                self.treeview.set_sensitive(True)
                self.button_cancel.set_sensitive(True)
                self.set_subscribe_button_sensitive()
            util.idle_add(download_thread_finished)

        threading.Thread(target=download_thread_func).start()

    def on_select_all_button_clicked(self, widget):
        self.do_select(all=True)

    def on_select_none_button_clicked(self, widget):
        self.do_select(all=False)

    def on_invert_selection_button_clicked(self, widget):
        self.do_select(invert=True)

    def do_select(self, all=False, invert=False):
        model = self.treeview.get_model()
        if model is not None:
            for row in model:
                if invert:
                    row[OpmlListModel.C_SELECTED] = \
                            not row[OpmlListModel.C_SELECTED]
                else:
                    row[OpmlListModel.C_SELECTED] = all
        self.set_subscribe_button_sensitive()

    def set_subscribe_button_sensitive(self):
        model = self.treeview.get_model()
        if model is not None:
            for row in model:
                if row[OpmlListModel.C_SELECTED]:
                    self.button_subscribe.set_sensitive(True)
                    return
        self.button_subscribe.set_sensitive(False)

    def on_subscribe_button_clicked(self, widget, *args):
        channel_urls = self.get_selected_channels()
        self.main_window.destroy()

        # add channels that have been selected
        if self.add_urls_callback is not None:
            self.add_urls_callback(channel_urls)

    def on_close_button_clicked(self, widget):
        self.main_window.destroy()




#!/usr/bin/python
# Simple script to test gPodder's "pill" pixbuf implementation
# Thomas Perl <thp.io/about>; 2009-09-13

import sys
sys.path.insert(0, 'src')

import gtk

from gpodder.gtkui.draw import draw_pill_pixbuf

def gen(x, y):
    if y >= x:
        pixbuf = draw_pill_pixbuf(str(x), str(y))
        return gtk.image_new_from_pixbuf(pixbuf)
    else:
        pixbuf = draw_pill_pixbuf('0', '0')
        return gtk.image_new_from_pixbuf(pixbuf)

w = gtk.Window()
w.connect('destroy', gtk.main_quit)
v = gtk.VBox()
w.add(v)
for y in xrange(15):
    h = gtk.HBox()
    h.set_homogeneous(True)
    v.add(h)
    for x in xrange(15):
        h.add(gen(x, y))
w.set_default_size(400, 400)
w.show_all()
gtk.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# gPodder - A media aggregator and podcast client
# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
#
# gPodder is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# gPodder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""Win32 Launcher script for gPodder

This is only used for the Win32 version of gPodder
and will set up the environment to find all files
and then start up the gPodder GUI.

Thomas Perl <thp@gpodder.org>; 2009-05-09
"""

import sys
import os
import os.path

import gettext

if __name__ == '__main__':
    prefix = os.path.abspath(os.path.normpath('.'))
    data_dir = os.path.join(prefix, 'data')

    locale_dir = os.path.join(data_dir, 'locale')
    ui_folder = os.path.join(data_dir, 'ui')
    credits_file = os.path.join(data_dir, 'credits.txt')
    images_folder = os.path.join(data_dir, 'images')
    icon_file = os.path.join(data_dir, 'gpodder.png')

    # Set up the path to translation files
    gettext.bindtextdomain('gpodder', locale_dir)

    import gpodder

    if not gpodder.win32:
        print >>sys.stderr, 'This launcher is only for Win32.'
        sys.exit(1)

    # Enable i18n for gPodder translations
    _ = gpodder.gettext

    # Set up paths to folder with GtkBuilder files and gpodder.svg
    gpodder.ui_folders.append(ui_folder)
    gpodder.ui_folders.append(os.path.join(ui_folder, 'desktop'))
    gpodder.icon_file = icon_file
    gpodder.credits_file = credits_file
    gpodder.images_folder = images_folder
    gpodder.ui.desktop = True

    # Portable version support
    if (os.path.exists('downloads') and os.path.exists('config')) or \
       not os.path.exists(os.path.expanduser('~/.config/gpodder')):
        home = os.path.join(os.getcwd(), 'config')
        if not os.path.exists(home):
            os.mkdir(home)
        gpodder.home = home
        gpodder.subscription_file = os.path.join(home, 'channels.opml')
        gpodder.config_file = os.path.join(home, 'gpodder.conf')
        gpodder.database_file = os.path.join(home, 'database.sqlite')
        download_dir = os.path.join(os.getcwd(), 'downloads')
        os.environ['GPODDER_DOWNLOAD_DIR'] = download_dir

    from gpodder import gui

    class options(object):
        subscribe = None

    gui.main(options)




# -*- coding: utf-8 -*-
#
# gPodder - A media aggregator and podcast client
# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
#
# gPodder is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# gPodder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import hildon
import gobject

class EditToolbarDeluxe(hildon.EditToolbar):
    """HildonEditToolbar with sensitivity and AppMenu action

    * Public method to set the button sensitivity
    * Open the parent window's AppMenu when touching titlebar

    (Enhancement requests filed as Maemo bugs #5166 and #5167)
    """
    def __init__(self, label_text, button_text):
        hildon.EditToolbar.__init__(self, label_text, button_text)

        alignment, separator, close_button = self.get_children()
        hbox = alignment.get_child()
        label, image, button = hbox.get_children()
        self._action_button = button

        expand, fill, padding, pack_type = hbox.query_child_packing(label)
        event_box = gtk.EventBox()
        event_box.connect('expose-event', self._on_expose_event)
        label.reparent(event_box)
        event_box.connect('button-release-event', self._on_label_clicked)
        hbox.add(event_box)
        hbox.reorder_child(event_box, 0)
        hbox.set_child_packing(event_box, expand, fill, padding, pack_type)
        self._label = label

    def set_button_sensitive(self, sensitivity):
        self._action_button.set_sensitive(sensitivity)

    def _on_expose_event(self, widget, event):
        # Based on hildon_edit_toolbar_expose() in hildon-edit-toolbar.c
        # in order to get the same style background for the EventBox widget
        style = self.get_style()
        style.paint_flat_box(widget.window, \
                gtk.STATE_NORMAL, \
                gtk.SHADOW_NONE, \
                event.area, widget, 'edit-toolbar', \
                0, 0, \
                widget.allocation.width, widget.allocation.height)
        # Propagate the expose event to the child (our label)
        child = widget.get_child()
        widget.propagate_expose(child, event)
        return True

    def _on_label_clicked(self, widget, event):
        parent = self.get_parent()
        app_menu = parent.get_app_menu()
        if app_menu is not None:
            app_menu.popup(parent)


class FancyProgressBar(object):
    """An animated progress bar with cancel button (bling bling!)

    This is a helper class coordinating some widgets that are needed for the
    feed update progress bar. It takes care of hooking up and resizing an event
    box (to be used in the main window), of creating the progress bar and
    cancel button widgets and of controlling the show/hide animation as well
    as reacting to the click on the cancel button (by means of the event box).

    In short, this provides all the necessary hooks to create a nice, sliding
    progress bar using hildon.AnimationActor while hiding the plumbing.

    Usage:

    Create a new object, where parent is the hildon.StackableWindow on which
    this progress bar should be shown. The on_cancel should be a callback with
    one parameter (will be the FancyProgressBar instance from which it is
    called) or None. It will be called when the cancel button is clicked.

    The using code should then take the "event_box" attribute and place it at
    the bottom of the parent window (e.g. as last item in a gtk.VBox). After
    that, calling "show()" and "hide()" will take care of hiding and showing
    both the "contents" (progress bar) and the event box. The contents will be
    slided in and out using a nice animation.

    You can get the progress bar from the "progress_bar" attribute and use it
    in your code to update the contents (it's a simple gtk.ProgressBar object).
    """

    SHOW, HIDE = range(2)
    FREMANTLE_TITLE_BAR_SIZE = 56
    STEP = 10

    def __init__(self, parent, on_cancel=None):
        self.parent = parent
        self.on_cancel = on_cancel
        self.state = FancyProgressBar.HIDE
        self.offset = float(1)

        self.progress_bar = gtk.ProgressBar()
        image = gtk.image_new_from_icon_name('general_stop', \
                gtk.ICON_SIZE_LARGE_TOOLBAR)
        self.cancel_button = gtk.ToolButton(image)
        self.hbox_animation_actor = gtk.HBox()
        self.hbox_inner = gtk.HBox()
        self.hbox_animation_actor.pack_start(self.hbox_inner)
        self.hbox_animation_actor.child_set(self.hbox_inner, 'padding', 12)
        self.hbox_inner.pack_start(self.progress_bar)
        self.hbox_inner.pack_start(self.cancel_button, False)
        self.animation_actor = hildon.AnimationActor()
        self.animation_actor.set_parent(self.parent)
        self.animation_actor.add(self.hbox_animation_actor)
        self.animation_actor.show_all()
        self.height, self.width = self.cancel_button.size_request()
        self.event_box = gtk.EventBox()
        self.event_box.connect('button-release-event', self.on_button_release)
        self.event_box.set_size_request(-1, self.height)
        self.offset = float(self.height)
        self.relayout()

    def on_button_release(self, widget, event):
        if event.x > self.cancel_button.get_allocation().x:
            if self.on_cancel is not None:
                self.on_cancel(self)

    def relayout(self):
        width, height = self.parent.get_size()
        self.animation_actor.resize(width, self.height)
        x, y = 0, FancyProgressBar.FREMANTLE_TITLE_BAR_SIZE
        y += height + int(self.offset)
        y -= self.height
        self.animation_actor.set_position(x, y)
        return False

    def show(self):
        if self.state != FancyProgressBar.SHOW:
            self.state = FancyProgressBar.SHOW
            self.hbox_animation_actor.show()
            self.offset = float(self.height)
            gobject.timeout_add(FancyProgressBar.STEP, self.on_timeout)

    def hide(self):
        if self.state != FancyProgressBar.HIDE:
            self.event_box.hide()
            self.state = FancyProgressBar.HIDE
            self.offset = float(1.)
            gobject.timeout_add(FancyProgressBar.STEP, self.on_timeout)

    def on_timeout(self):
        result = True

        if self.state == FancyProgressBar.SHOW:
            self.offset *= .9
            if self.offset < 2:
                self.event_box.show()
                self.offset = 0.
                result = False
        elif self.state == FancyProgressBar.HIDE:
            self.offset *= 1.1
            if self.offset >= self.height:
                self.offset = float(self.height)
                self.hbox_animation_actor.hide()
                result = False

        self.relayout()

        return result




# -*- coding: utf-8 -*-
#
# gPodder - A media aggregator and podcast client
# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
#
# gPodder is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# gPodder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


#
#  draw.py -- Draw routines for gPodder-specific graphics
#  Thomas Perl <thp@perli.net>, 2007-11-25
#

import gpodder

import gtk
import pango
import pangocairo
import cairo
import StringIO
import math


class TextExtents(object):
    def __init__(self, ctx, text):
        tuple = ctx.text_extents(text)
        (self.x_bearing, self.y_bearing, self.width, self.height, self.x_advance, self.y_advance) = tuple


RRECT_LEFT_SIDE = 1
RRECT_RIGHT_SIDE = 2

def draw_rounded_rectangle(ctx, x, y, w, h, r=10, left_side_width = None, sides_to_draw=0, close=False):
    assert left_side_width is not None

    x = int(x)
    offset = 0
    if close: offset = 0.5

    if sides_to_draw & RRECT_LEFT_SIDE:
        ctx.move_to(x+int(left_side_width)-offset, y+h)
        ctx.line_to(x+r, y+h)
        ctx.curve_to(x, y+h, x, y+h, x, y+h-r)
        ctx.line_to(x, y+r)
        ctx.curve_to(x, y, x, y, x+r, y)
        ctx.line_to(x+int(left_side_width)-offset, y)
        if close:
            ctx.line_to(x+int(left_side_width)-offset, y+h)

    if sides_to_draw & RRECT_RIGHT_SIDE:
        ctx.move_to(x+int(left_side_width)+offset, y)
        ctx.line_to(x+w-r, y)
        ctx.curve_to(x+w, y, x+w, y, x+w, y+r)
        ctx.line_to(x+w, y+h-r)
        ctx.curve_to(x+w, y+h, x+w, y+h, x+w-r, y+h)
        ctx.line_to(x+int(left_side_width)+offset, y+h)
        if close:
            ctx.line_to(x+int(left_side_width)+offset, y)


def rounded_rectangle(ctx, x, y, width, height, radius=4.):
    """Simple rounded rectangle algorithmn

    http://www.cairographics.org/samples/rounded_rectangle/
    """
    degrees = math.pi / 180.
    ctx.new_sub_path()
    if width > radius:
        ctx.arc(x + width - radius, y + radius, radius, -90. * degrees, 0)
        ctx.arc(x + width - radius, y + height - radius, radius, 0, 90. * degrees)
        ctx.arc(x + radius, y + height - radius, radius, 90. * degrees, 180. * degrees)
        ctx.arc(x + radius, y + radius, radius, 180. * degrees, 270. * degrees)
    ctx.close_path()


def draw_text_box_centered(ctx, widget, w_width, w_height, text, font_desc=None, add_progress=None):
    style = widget.rc_get_style()
    text_color = style.text[gtk.STATE_PRELIGHT]
    red, green, blue = text_color.red, text_color.green, text_color.blue
    text_color = [float(x)/65535. for x in (red, green, blue)]
    text_color.append(.5)

    if font_desc is None:
        font_desc = style.font_desc
        font_desc.set_size(14*pango.SCALE)

    pango_context = widget.create_pango_context()
    layout = pango.Layout(pango_context)
    layout.set_font_description(font_desc)
    layout.set_text(text)
    width, height = layout.get_pixel_size()

    ctx.move_to(w_width/2-width/2, w_height/2-height/2)
    ctx.set_source_rgba(*text_color)
    ctx.show_layout(layout)

    # Draw an optional progress bar below the text (same width)
    if add_progress is not None:
        bar_height = 10
        ctx.set_source_rgba(*text_color)
        ctx.set_line_width(1.)
        rounded_rectangle(ctx, w_width/2-width/2-.5, w_height/2+height-.5, width+1, bar_height+1)
        ctx.stroke()
        rounded_rectangle(ctx, w_width/2-width/2, w_height/2+height, int(width*add_progress)+.5, bar_height)
        ctx.fill()


def draw_text_pill(left_text, right_text, x=0, y=0, border=2, radius=14, font_desc=None):
    if gpodder.ui.fremantle:
        border += 3
    # Create temporary context to calculate the text size
    ctx = cairo.Context(cairo.ImageSurface(cairo.FORMAT_ARGB32, 1, 1))

    # Use GTK+ style of a normal Button
    widget = gtk.Label()
    style = widget.rc_get_style()

    x_border = border*2

    if font_desc is None:
        font_desc = style.font_desc
        font_desc.set_weight(pango.WEIGHT_BOLD)

    pango_context = widget.create_pango_context()
    layout_left = pango.Layout(pango_context)
    layout_left.set_font_description(font_desc)
    layout_left.set_text(left_text)
    layout_right = pango.Layout(pango_context)
    layout_right.set_font_description(font_desc)
    layout_right.set_text(right_text)

    width_left, height_left = layout_left.get_pixel_size()
    width_right, height_right = layout_right.get_pixel_size()

    text_height = max(height_left, height_right)

    image_height = int(y+text_height+border*2)
    image_width = int(x+width_left+width_right+x_border*4)
    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, image_width, image_height)

    ctx = pangocairo.CairoContext(cairo.Context(surface))

    if left_text == '0':
        left_text = None
    if right_text == '0':
        right_text = None

    left_side_width = width_left + x_border*2
    right_side_width = width_right + x_border*2

    rect_width = left_side_width + right_side_width
    rect_height = text_height + border*2
    if left_text is not None:
        draw_rounded_rectangle(ctx,x,y,rect_width,rect_height,radius, left_side_width, RRECT_LEFT_SIDE, right_text is None)
        linear = cairo.LinearGradient(x, y, x+left_side_width/2, y+rect_height/2)
        linear.add_color_stop_rgba(0, .8, .8, .8, .5)
        linear.add_color_stop_rgba(.4, .8, .8, .8, .7)
        linear.add_color_stop_rgba(.6, .8, .8, .8, .6)
        linear.add_color_stop_rgba(.9, .8, .8, .8, .8)
        linear.add_color_stop_rgba(1, .8, .8, .8, .9)
        ctx.set_source(linear)
        ctx.fill()
        xpos, ypos, width_left, height = x+1, y+1, left_side_width, rect_height-2
        if right_text is None:
            width_left -= 2
        draw_rounded_rectangle(ctx, xpos, ypos, rect_width, height, radius, width_left, RRECT_LEFT_SIDE, right_text is None)
        ctx.set_source_rgba(1., 1., 1., .3)
        ctx.set_line_width(1)
        ctx.stroke()
        draw_rounded_rectangle(ctx,x,y,rect_width,rect_height,radius, left_side_width, RRECT_LEFT_SIDE, right_text is None)
        ctx.set_source_rgba(.2, .2, .2, .6)
        ctx.set_line_width(1)
        ctx.stroke()

        ctx.move_to(x+x_border, y+1+border)
        ctx.set_source_rgba( 0, 0, 0, 1)
        ctx.show_layout(layout_left)
        ctx.move_to(x-1+x_border, y+border)
        ctx.set_source_rgba( 1, 1, 1, 1)
        ctx.show_layout(layout_left)

    if right_text is not None:
        draw_rounded_rectangle(ctx, x, y, rect_width, rect_height, radius, left_side_width, RRECT_RIGHT_SIDE, left_text is None)
        linear = cairo.LinearGradient(x+left_side_width, y, x+left_side_width+right_side_width/2, y+rect_height)
        linear.add_color_stop_rgba(0, .2, .2, .2, .9)
        linear.add_color_stop_rgba(.4, .2, .2, .2, .8)
        linear.add_color_stop_rgba(.6, .2, .2, .2, .6)
        linear.add_color_stop_rgba(.9, .2, .2, .2, .7)
        linear.add_color_stop_rgba(1, .2, .2, .2, .5)
        ctx.set_source(linear)
        ctx.fill()
        xpos, ypos, width, height = x, y+1, rect_width-1, rect_height-2
        if left_text is None:
            xpos, width = x+1, rect_width-2
        draw_rounded_rectangle(ctx, xpos, ypos, width, height, radius, left_side_width, RRECT_RIGHT_SIDE, left_text is None)
        ctx.set_source_rgba(1., 1., 1., .3)
        ctx.set_line_width(1)
        ctx.stroke()
        draw_rounded_rectangle(ctx, x, y, rect_width, rect_height, radius, left_side_width, RRECT_RIGHT_SIDE, left_text is None)
        ctx.set_source_rgba(.1, .1, .1, .6)
        ctx.set_line_width(1)
        ctx.stroke()

        ctx.move_to(x+left_side_width+x_border, y+1+border)
        ctx.set_source_rgba( 0, 0, 0, 1)
        ctx.show_layout(layout_right)
        ctx.move_to(x-1+left_side_width+x_border, y+border)
        ctx.set_source_rgba( 1, 1, 1, 1)
        ctx.show_layout(layout_right)

    return surface


def draw_pill_pixbuf(left_text, right_text):
    return cairo_surface_to_pixbuf(draw_text_pill(left_text, right_text))


def cairo_surface_to_pixbuf(s):
    """
    Converts a Cairo surface to a Gtk Pixbuf by
    encoding it as PNG and using the PixbufLoader.
    """
    sio = StringIO.StringIO()
    try:
        s.write_to_png(sio)
    except:
        # Write an empty PNG file to the StringIO, so
        # in case of an error we have "something" to
        # load. This happens in PyCairo < 1.1.6, see:
        # http://webcvs.cairographics.org/pycairo/NEWS?view=markup
        # Thanks to Chris Arnold for reporting this bug
        sio.write('iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAAAXNSR0IArs4c6QAAAAZiS0dEAP8A\n/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9cMEQkqIyxn3RkAAAAZdEVYdENv\nbW1lbnQAQ3JlYXRlZCB3aXRoIEdJTVBXgQ4XAAAADUlEQVQI12NgYGBgAAAABQABXvMqOgAAAABJ\nRU5ErkJggg==\n'.decode('base64'))

    pbl = gtk.gdk.PixbufLoader()
    pbl.write(sio.getvalue())
    pbl.close()

    pixbuf = pbl.get_pixbuf()
    return pixbuf

def progressbar_pixbuf(width, height, percentage):
    COLOR_BG = (.4, .4, .4, .4)
    COLOR_FG = (.2, .9, .2, 1.)
    COLOR_FG_HIGH = (1., 1., 1., .5)
    COLOR_BORDER = (0., 0., 0., 1.)

    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
    ctx = cairo.Context(surface)

    padding = int(float(width)/8.0)
    bar_width = 2*padding
    bar_height = height - 2*padding
    bar_height_fill = bar_height*percentage

    # Background
    ctx.rectangle(padding, padding, bar_width, bar_height)
    ctx.set_source_rgba(*COLOR_BG)
    ctx.fill()

    # Foreground
    ctx.rectangle(padding, padding+bar_height-bar_height_fill, bar_width, bar_height_fill)
    ctx.set_source_rgba(*COLOR_FG)
    ctx.fill()
    ctx.rectangle(padding+bar_width/3, padding+bar_height-bar_height_fill, bar_width/4, bar_height_fill)
    ctx.set_source_rgba(*COLOR_FG_HIGH)
    ctx.fill()

    # Border
    ctx.rectangle(padding-.5, padding-.5, bar_width+1, bar_height+1)
    ctx.set_source_rgba(*COLOR_BORDER)
    ctx.set_line_width(1.)
    ctx.stroke()

    return cairo_surface_to_pixbuf(surface)

